package com.molino.mills.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;

import com.molino.mills.model.Board;
import com.molino.mills.model.Game;
import com.molino.mills.model.pos.XYBoardPosition;

/**
 * @author  Emilio
 */
public class BoardComponent extends JComponent
{
    /**
	 * @uml.property  name="gameController"
	 * @uml.associationEnd  
	 */
    private GameController gameController;
    /**
	 * @uml.property  name="pieceDiameter"
	 */
    private int pieceDiameter = 16;
    private int xc; 
	private int yc;
	private int wc;
	private int hc;
    /**
	 * @uml.property  name="pieces"
	 * @uml.associationEnd  multiplicity="(0 -1)"
	 */
    private PieceComponent[] pieces = new PieceComponent[24];
    /**
	 * @uml.property  name="dotSize"
	 */
    private int dotSize = 5;
    /**
	 * @uml.property  name="showIndices"
	 */
    private boolean showIndices;
    /**
	 * @uml.property  name="showCoordinates"
	 */
    private boolean showCoordinates;
    /**
	 * @uml.property  name="pieceListener"
	 * @uml.associationEnd  
	 */
    private PieceListener pieceListener;
    /**
	 * @uml.property  name="indexFont"
	 */
    private Font indexFont;
    /**
	 * @uml.property  name="coordFont"
	 */
    private Font coordFont;
    /**
	 * @uml.property  name="indexColor"
	 */
    private Color indexColor;
    /**
	 * @uml.property  name="coordColor"
	 */
    private Color coordColor;



    public BoardComponent()
    {
        super();
        indexFont = new Font("Verdana,Arial,SansSerif", Font.PLAIN, 9);
        coordFont = new Font("Verdana,Arial,SansSerif", Font.BOLD, 12);
        indexColor = new Color(0xAA, 0xAA, 0xAA);
        coordColor = new Color(0x00, 0x00, 0x00);
    }

    public void initGUI()
    {
        removeAll();
        
        int r = pieceDiameter / 2;
        if (showCoordinates) 
            setBorder(BorderFactory.createEmptyBorder(r+14, r+14, r+14, r+14));
        else
            setBorder(BorderFactory.createEmptyBorder(r, r, r, r));
        
        BoardGridComponent bgc = new BoardGridComponent();
        bgc.setLocation(0,0);
        bgc.setSize(getSize());        
        bgc.setPreferredSize(getPreferredSize());
        bgc.setBorder(getBorder());
        add(bgc);
        
        refreshDimensions();
        
        XYBoardPosition pos = new XYBoardPosition();
        JLabel label;
        
        if ( showCoordinates )
        {
            for (int i=0; i<=6; i++)
            {
                int x = xc + (i * wc / 6) - r + 5;
                int y = yc + (i * hc / 6) - r + 4;
                
                label = new JLabel(Integer.toString(i+1));
                add(label);
                label.setFont(coordFont);
                label.setForeground(coordColor);
                label.setBounds(0, y, 30, 10);
                
                label = new JLabel(Character.toString((char) (i+'A')));
                add(label);
                label.setFont(coordFont);
                label.setForeground(coordColor);
                label.setBounds(x, 0, 30, 10);                                
            }
        }

        for (int i = 0; i <= 23; i++)
        {
            pos.setPosition(i);
            int x = xc + ((pos.getXPos()+3) * wc / 6) - r;
            int y = yc + ((pos.getYPos()+3) * hc / 6) - r;
           
            if (showIndices)
            {
                int x2 = x;
                int y2 = y;

                label = new JLabel(Integer.toString(i));
                add(label);
                label.setFont(indexFont);
                label.setForeground(indexColor);

                if (pos.getXPos() != 3) x2 += 12; 

                if (pos.getYPos() == 3)
                    y2 -= 2;
                else
                    y2 += 12;

                label.setBounds(x2, y2, 30, 10);
            }
            

            pieces[i] = new PieceComponent();            
            if (pieceListener != null) 
                pieces[i].addDragListener(pieceListener);
            add(pieces[i]);
            pieces[i].setPosition(i);
            pieces[i].setLocation(x, y);
            setComponentZOrder(pieces[i], 0);
        }
    }

    public XYBoardPosition getPositionAt(Point p)
    {
        return getPositionAt((int) p.getX(), (int) p.getY());
    }

    public XYBoardPosition getPositionAt(int x, int y)
    {
        int row = (x - xc + (wc / 12)) / (wc / 6);
        int col = (y - yc + (hc / 12)) / (hc / 6);

        return new XYBoardPosition(row - 3, col - 3);
    }

    /**
     * 
     */
    private void refreshDimensions()
    {
        Insets insets = getInsets();
        xc = insets.left;
        yc = insets.top;
        int size = Math.min(getWidth(), getHeight());
        wc = size - insets.left - insets.right - 1;
        hc = size - insets.top - insets.bottom - 1;
    }

    /**
	 * @param pdl
	 * @uml.property  name="pieceListener"
	 */
    public void setPieceListener(PieceListener pdl)
    {
        this.pieceListener = pdl;
    }

    public void refreshPieceColors()
    {
        Game game = gameController.getCurrentGame();
        Board board = game.state.board;
        Color color1 = game.getPlayer(0).getColor();
        Color color2 = game.getPlayer(1).getColor();
        for (int i = 0; i <= 23; i++)
        {           
            if (board.isPieceAt(i, 0))
            {
                pieces[i].setColor(color1);
                pieces[i].setOwner(0);
            }
            else if (board.isPieceAt(i, 1))
            {
                pieces[i].setColor(color2);
                pieces[i].setOwner(1);
            }
            else 
            {
                pieces[i].setOwner(-1);
                pieces[i].setColor(null);

            }
        }
    }

    /**
	 * @return
	 * @uml.property  name="pieceDiameter"
	 */
    public int getPieceDiameter()
    {
        return pieceDiameter;
    }

    /**
	 * @param pieceDiameter
	 * @uml.property  name="pieceDiameter"
	 */
    public void setPieceDiameter(int pieceDiameter)
    {
        this.pieceDiameter = pieceDiameter;
    }

    /**
	 * @return
	 * @uml.property  name="dotSize"
	 */
    public int getDotSize()
    {
        return dotSize;
    }

    /**
	 * @param dotSize
	 * @uml.property  name="dotSize"
	 */
    public void setDotSize(int dotSize)
    {
        this.dotSize = dotSize;
    }

    /**
	 * @return
	 * @uml.property  name="gameController"
	 */
    public GameController getGameController()
    {
        return gameController;
    }

    /**
	 * @param gameController
	 * @uml.property  name="gameController"
	 */
    public void setGameController(GameController gameController)
    {
        this.gameController = gameController;
    }

    /**
	 * @return
	 * @uml.property  name="showCoordinates"
	 */
    public boolean isShowCoordinates()
    {
        return showCoordinates;
    }

    /**
	 * @param showCoordinates
	 * @uml.property  name="showCoordinates"
	 */
    public void setShowCoordinates(boolean showCoordinates)
    {
        this.showCoordinates = showCoordinates;
    }

    /**
	 * @return
	 * @uml.property  name="showIndices"
	 */
    public boolean isShowIndices()
    {
        return showIndices;
    }

    /**
	 * @param showNumbers
	 * @uml.property  name="showIndices"
	 */
    public void setShowIndices(boolean showNumbers)
    {
        this.showIndices = showNumbers;
    }

    /**
	 * @return
	 * @uml.property  name="pieceListener"
	 */
    public PieceListener getPieceListener()
    {
        return pieceListener;
    }

    /**
	 * @return
	 * @uml.property  name="coordColor"
	 */
    public Color getCoordColor()
    {
        return coordColor;
    }

    /**
	 * @param coordColor
	 * @uml.property  name="coordColor"
	 */
    public void setCoordColor(Color coordColor)
    {
        this.coordColor = coordColor;
    }

    /**
	 * @return
	 * @uml.property  name="coordFont"
	 */
    public Font getCoordFont()
    {
        return coordFont;
    }

    /**
	 * @param coordFont
	 * @uml.property  name="coordFont"
	 */
    public void setCoordFont(Font coordFont)
    {
        this.coordFont = coordFont;
    }

    /**
	 * @return
	 * @uml.property  name="indexColor"
	 */
    public Color getIndexColor()
    {
        return indexColor;
    }

    /**
	 * @param indexColor
	 * @uml.property  name="indexColor"
	 */
    public void setIndexColor(Color indexColor)
    {
        this.indexColor = indexColor;
    }

    /**
	 * @return
	 * @uml.property  name="indexFont"
	 */
    public Font getIndexFont()
    {
        return indexFont;
    }

    /**
	 * @param indexFont
	 * @uml.property  name="indexFont"
	 */
    public void setIndexFont(Font indexFont)
    {
        this.indexFont = indexFont;
    }
}