package com.molino.mills.gui;

import java.awt.Color;

import javax.swing.JOptionPane;

import com.molino.mills.agent.ComputerPlayer;
import com.molino.mills.model.Game;
import com.molino.mills.model.GameListener;
import com.molino.mills.model.Player;
import com.molino.mills.model.event.NewGameEvent;
import com.molino.mills.model.move.CaptureMove;
import com.molino.mills.model.move.FromToMove;
import com.molino.mills.model.move.Move;
import com.molino.mills.model.move.MoveAndCaptureMove;
import com.molino.mills.model.move.PlaceMove;
import com.molino.mills.model.pos.ColRowBoardPosition;

/**
 * @author  Emilio
 */
public class GameController
{
    /**
	 * @uml.property  name="currentGame"
	 * @uml.associationEnd  
	 */
    Game currentGame;
    /**
	 * @uml.property  name="gameListener"
	 * @uml.associationEnd  
	 */
    GameListener gameListener; 
    private static final String[] playerOptions = {"Ordenador", "Humano"};
    private static final String[] colorOptions = {"Negras", "Blancas"};
    private static final Color[] colors = {Color.BLACK, Color.WHITE};
    private static final String[] altColorOptions = {"Blue", "Red"};
    private static final Color[] altColors = {Color.BLUE, Color.RED};
    
    public GameController()
    {
    }

    public boolean testnewGame()
    {
        Player[] players = new Player[2];
        players[0] = new ComputerPlayer();
//        players[0] = new HumanPlayer();
//        players[1] = new ComputerPlayer();
        players[1] = new HumanPlayer();
        players[0].setColor(altColors[0]);
        players[0].setColorName(altColorOptions[0]);
        players[1].setColor(altColors[1]);
        players[1].setColorName(altColorOptions[1]);
 
        currentGame = new Game(players);
        currentGame.addGameListener(gameListener);
        currentGame.notifyListeners(new NewGameEvent(currentGame));
         
        return true;
    }
    
    public boolean newGame()
    {        
        if (isGameActive())
        {
            if (JOptionPane.showConfirmDialog(null, 
                    "Un juego esta en curso!Abortar juego?", 
                    "Confirmar", JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
                return false;

            currentGame.interrupt();
            currentGame = null;
        }

        Player[] players = new Player[2];
        players[0] = JOptionPane.showOptionDialog(null, 
                        "Quien controlara al jugador 1?", "Opciones juego nuevo", 
                        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, 
                        null, playerOptions, playerOptions[0]) == JOptionPane.YES_OPTION ? new ComputerPlayer() : new HumanPlayer();
        int player1ColorOption = JOptionPane.showOptionDialog(null, 
                            "Cual sera el color del jugador 1?", "Opciones juego nuevo", 
                            JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, 
                            null, colorOptions, colorOptions[1]);
        
        
        players[1] = JOptionPane.showOptionDialog(null, 
                "Quien controlara al jugador 2?", "Opciones juego nuevo", 
                JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, 
                null, playerOptions, playerOptions[1]) == JOptionPane.YES_OPTION ? new ComputerPlayer() : new HumanPlayer();
                
        players[0].setColor(player1ColorOption==JOptionPane.YES_OPTION ? colors[0] : colors[1]);
        players[0].setColorName(player1ColorOption==JOptionPane.YES_OPTION ? colorOptions[0] : colorOptions[1]);
        players[1].setColor(player1ColorOption!=JOptionPane.YES_OPTION ? colors[0] : colors[1]);
        players[1].setColorName(player1ColorOption!=JOptionPane.YES_OPTION ? colorOptions[0] : colorOptions[1]);

        currentGame = new Game(players);
        currentGame.addGameListener(gameListener);
        currentGame.notifyListeners(new NewGameEvent(currentGame));
        String s = JOptionPane.showInputDialog(null, 
                "Por favor introduzca un limite de tiempo,", "10");
        currentGame.setTimeLimit(Integer.parseInt(s));
        return true;
    }

    /**
	 * @return
	 * @uml.property  name="currentGame"
	 */
    public Game getCurrentGame()
    {
        return currentGame;
    }

    public boolean isGameActive()
    {
        return currentGame != null && currentGame.isAlive();
    }

    private HumanPlayer getHumanPlayer()
    {
        if (!isGameActive())
        {
            if (JOptionPane.showConfirmDialog(null, "Ningun juego en progreso . Comenzar juego nuevo?", "Game Over!", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
            {
                newGame();
            }

            return null;
        }
        
        Player player = currentGame.getCurrentPlayer();
        if (!(player instanceof HumanPlayer))
        {
            JOptionPane.showMessageDialog(null, "Shhh... Estoy pensando!!!  Espera que termine...", "Ocupado...", JOptionPane.WARNING_MESSAGE);

            return null;
        }
        else
            return (HumanPlayer) player;
    }

    public void attemptMove(Move move, boolean alertOnBadMove)
    {
        HumanPlayer hplayer = getHumanPlayer();

        if (hplayer != null) attemptMove(hplayer, move, alertOnBadMove);
    }

    /**
     * @param hplayer
     * @param move
     * @param alertOnBadMove 
     */
    private void attemptMove(HumanPlayer hplayer, Move move, boolean alertOnBadMove)
    {
        if (!currentGame.isLegalMove(move))
        {
            if (alertOnBadMove) JOptionPane.showMessageDialog(null, "Movimiento ilegal.", "Cheat Detector v1.0", JOptionPane.WARNING_MESSAGE);
            return;
        }

        hplayer.setNextMove(move);
    }

    public void attemptMove(String txt)
    {
        HumanPlayer hplayer = getHumanPlayer();

        if (hplayer == null) return;
        ColRowBoardPosition pos = null, dest = null, capture = null;
        String[] moves = txt.split(","); 
        String[] fromto = moves[0].split("-");
        
        if (fromto.length>=1) pos = new ColRowBoardPosition(fromto[0].trim());
        if (fromto.length==2) dest = new ColRowBoardPosition(fromto[1].trim());
        if (moves.length>=2)  capture = new ColRowBoardPosition(moves[1].trim());
        
        if ( moves.length > 2 ||
             fromto.length > 2 ||
             pos == null || !pos.isValid() || 
            (dest != null && !dest.isValid()) || 
            (capture != null && !capture.isValid()))
        {
            JOptionPane.showMessageDialog(null, "Por favor introduzca movimiento valido.", "Movimiento invalido!", JOptionPane.WARNING_MESSAGE);
            
            return;
        }

        Move move;
        if (dest == null)
            if ( currentGame.state.awaitingCapture > 0 )
                move = new CaptureMove(pos.getPosition());
            else
                move = new PlaceMove(pos.getPosition());
        else
            move = new FromToMove(pos.getPosition(), dest.getPosition());

        if ( capture != null )
            move = new MoveAndCaptureMove(move, new CaptureMove(capture.getPosition()));
        
        attemptMove(hplayer, move, true);
        
    }

    /**
	 * @return
	 * @uml.property  name="gameListener"
	 */
    public GameListener getGameListener()
    {
        return gameListener;
    }

    /**
	 * @param gameListener
	 * @uml.property  name="gameListener"
	 */
    public void setGameListener(GameListener gameListener)
    {
        this.gameListener = gameListener;
    }

    public void startGame()
    {
        currentGame.start();
    }

    public void takeBackMove()
    {
        currentGame.requestTakeBackMove();        
    }
}
