package com.molino.mills.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.MouseEvent;

import com.molino.mills.model.move.CaptureMove;
import com.molino.mills.model.move.FromToMove;
import com.molino.mills.model.move.PlaceMove;
import com.molino.mills.model.pos.XYBoardPosition;

/**
 * @author  Emilio
 */
public class PieceListener extends DragListener
{
    /**
	 * @uml.property  name="gameController"
	 * @uml.associationEnd  
	 */
    private GameController gameController;
    /**
	 * @uml.property  name="boardComponent"
	 * @uml.associationEnd  
	 */
    private BoardComponent boardComponent;

    public PieceListener()
    {
        super();
    }

    public PieceListener(Container dc, GameController gc)
    {
        setDragContainer(dc);
        gameController = gc;
    }

    @Override
    public void dragReleased(MouseEvent e)
    {
        PieceComponent pc = (PieceComponent) getComponent();
        Point dest = pc.getLocationOnScreen();

        super.dragReleased(e);

        int x = (int) (dest.getX() - boardComponent.getLocationOnScreen().getX());
        int y = (int) (dest.getY() - boardComponent.getLocationOnScreen().getY());
        XYBoardPosition pos = boardComponent.getPositionAt(x, y);
        if (pos.isValid())
        {
            if (pc.getPosition() == -1)
                gameController.attemptMove(new PlaceMove(pos.getPosition()), false);
            else
                gameController.attemptMove(new FromToMove(pc.getPosition(), pos.getPosition()), false);
        }
    }

    public void mouseClicked(MouseEvent e)
    {
        Object src = e.getSource();
        if (src instanceof PieceComponent)
        {
            PieceComponent pc = (PieceComponent) src;
            if (pc.getPosition() >= 0 && gameController.getCurrentGame().state.awaitingCapture > 0)
                gameController.attemptMove(new CaptureMove(pc.getPosition()), false);
            else
                gameController.attemptMove(new PlaceMove(pc.getPosition()), false);
        }
    }

    /**
	 * @return
	 * @uml.property  name="boardComponent"
	 */
    public BoardComponent getBoardComponent()
    {
        return boardComponent;
    }

    /**
	 * @param boardComponent
	 * @uml.property  name="boardComponent"
	 */
    public void setBoardComponent(BoardComponent boardComponent)
    {
        this.boardComponent = boardComponent;
    }

    @Override
    protected boolean isDraggable(Component c)
    {        
        if ( c instanceof PieceComponent )
        {
            PieceComponent pc = (PieceComponent) c;
            
            return pc.getColor()!=null && pc.getOwner()==gameController.currentGame.getCurrentPlayer().getPlayerNumber();
        }
        else
            return false;
    }
}
