package com.molino.mills.model;

import com.molino.mills.model.move.Move;

public class GameState
    implements Comparable
{

    /**
	 * @uml.property  name="board"
	 * @uml.associationEnd  
	 */
    public Board board;    
    public int turnsCompleted;
    public int currentPlayer;
    /**
	 * @uml.property  name="previous"
	 * @uml.associationEnd  
	 */
    public GameState previous;
    public int awaitingCapture;
    /**
	 * @uml.property  name="lastMove"
	 * @uml.associationEnd  
	 */
    public Move lastMove;
    
    // Features of board derived from current state:
    /**
	 * @uml.property  name="features"
	 * @uml.associationEnd  
	 */
    public BoardFeatures features;
    /**
	 * Current game stage (relative to current player)
	 * @uml.property  name="stage"
	 * @uml.associationEnd  
	 */
    public Stage stage;
    public int hashCode;
    
    /**
	 * @author   Emilio
	 */
    public enum Stage {/**
	 * @uml.property  name="oPENING"
	 * @uml.associationEnd  
	 */
    OPENING, /**
	 * @uml.property  name="mIDGAME"
	 * @uml.associationEnd  
	 */
    MIDGAME, /**
	 * @uml.property  name="eNDGAME"
	 * @uml.associationEnd  
	 */
    ENDGAME};

    public GameState()
    {
        board = new Board();
        features = new BoardFeatures(board);
        currentPlayer = 0;
        previous = null;
        turnsCompleted = 0;
        awaitingCapture = 0;
        stage = Stage.OPENING;
        hashCode = 0;
    }

    public GameState(Board board, GameState previous, Move lastMove)
    {
        this.board = board;
        this.previous = previous;
        this.features = new BoardFeatures(board);
        this.turnsCompleted = previous.turnsCompleted;
        this.currentPlayer = previous.currentPlayer;
        this.lastMove = lastMove;
        
        if ( hasNewMills() )
        {
            this.awaitingCapture = 1;
        }
        else
        {
            this.awaitingCapture = 0;
            this.turnsCompleted++;
            this.currentPlayer = 1-currentPlayer;            
        }
        
        updateStage(board);
        hashCode = calculateHashCode();
    }

    /**
     * @param board
     */
    protected void updateStage(Board board)
    {
        if (board.unplacedPieces[currentPlayer] > 0)
            stage = Stage.OPENING;
        else if (board.piecesLeft[currentPlayer] > 3)
            stage = Stage.MIDGAME;
        else
            stage = Stage.ENDGAME;
    }

    public int getNewMills()
    {
        return features.millposbbs[currentPlayer] & ~previous.features.millposbbs[currentPlayer]; 
    }
    
    public boolean hasNewMills()
    {
        return getNewMills() > 0;
    }
    
    public boolean hasOnlyMills(int player)
    {
        return features.millbbs[player] == board.bbs[player];
    }

    @Override
    public boolean equals(Object obj)
    {
        GameState other = (GameState) obj;
        // TODO Auto-generated method stub
        return this.currentPlayer==other.currentPlayer &&
               this.awaitingCapture==other.awaitingCapture &&
               this.board.equals(other.board);
    }

    @Override
    public int hashCode()
    {
        return hashCode;
    }

    public int calculateHashCode()
    {
        return (((board.hashCode()<<1) | currentPlayer) << 1) | awaitingCapture;
    }

    @Override
    public String toString()
    {
        return "lastMove="+lastMove+
               " turnsCompleted="+turnsCompleted+
               " player="+currentPlayer+
               " awaitingCapture="+awaitingCapture+
               " "+board;
    }

    public int compareTo(Object o)
    {
        return this.equals(o) ? 0 : -1;
    }
}
